unit ATStandard;
{
  ATComponents 1.01
  Copyright (C) 2000 by Arpad Toth
  All rights reserved.

  All contents of this file and all other files included in the ATComponents
  archive  are   Copyright (C) 2000 Arpad Toth.  Use and/or  distribution of
  them requires acceptance of the author.
}

interface

uses  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Stdctrls, Extctrls, Buttons;

type
 TTitleIcons = (idCustom, idForm, idApplication);

 TATGraphicCtrls = class(TGraphicControl)
 protected
  procedure CMTextChanged(var Msg: TMessage); message CM_TEXTCHANGED;
  procedure CMFontChanged(var Msg: TMessage); message CM_FONTCHANGED;
  procedure CMColorChanged(var Msg: TMessage); message CM_COLORCHANGED;
 public
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
 end;

 TATLabel = class(TATGraphicCtrls)
 private
  FSpace : word;
 protected
  procedure Paint; override;
//  procedure SetIndent(Value: integer);
 public
  constructor Create(AOwner: TComponent); override;
 published
  property Align;
  property Anchors;
  property Caption;
  property Constraints;
  property Color;
  property Font;
  property Space: word read FSpace write FSpace;
  property ShowHint;
  property ParentFont;
  property ParentColor;
  property Visible;
 end;

 TATTitlePanel = class(TGraphicControl)
 private
  FTitleLeft, FTitleTop, FCaptionLeft, FCaptionTop : integer;
  FTitleFont : TFont;
  FTitle     : string;
  FIcon      : TIcon;
  FIcontype  : TTitleIcons;
  procedure SetTypeIcon(Value: TTitleIcons);
  procedure SetTitle(Value: string);
  procedure SetTitleFont(Value: TFont);
  procedure SetIcon(Value: TIcon);
 protected
  procedure Paint; override;
 public
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
 published
  property Align;
  property Anchors;
  property Caption;
  property Constraints;
  property Color;
  property Font;
  property Visible;
  property Icon: TIcon read FIcon write SetIcon;
  property IconType: TTitleIcons read FIconType write SetTypeIcon;
  property Title: string read FTitle write SetTitle;
  property TitleFont: TFont read FTitleFont write SetTitleFont;
 end;

procedure Register;

implementation
//{$R ATStandard.dcr}

// TATGraphicCtrls
//------------------------------------------------------------------------------
constructor TATGraphicCtrls.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque];
  Color := clBtnFace;
  Width := 200;
  Height := 13;
end;

destructor TATGraphicCtrls.Destroy;
begin
  inherited;
end;
//------------------------------------------------------------------------------
procedure TATGraphicCtrls.CMTextChanged(var Msg: TMessage);
begin inherited; invalidate; end;
//------------------------------------------------------------------------------
procedure TATGraphicCtrls.CMFontChanged(var Msg: TMessage);
begin inherited; invalidate; end;
//------------------------------------------------------------------------------
procedure TATGraphicCtrls.CMColorChanged(var Msg: TMessage);
begin inherited; invalidate; end;
//------------------------------------------------------------------------------

// TATLabel
//------------------------------------------------------------------------------
constructor TATLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ParentColor := True;
  Height := 13;
  FSpace := 5;
end;

procedure TATLabel.Paint;
var
  R: TRect;
  Y: integer;
begin
  R := ClientRect;
  Canvas.Brush.Color := Self.Color;
  Canvas.FillRect(R);
  // Draw text
  R.Left := R.Left + FSpace;
  Canvas.Font.Assign(Self.Font);
  Canvas.Brush.Style := bsClear;
  DrawText(Canvas.Handle, PChar(Caption), Length(Caption), R, DT_SINGLELINE or DT_VCENTER);
  // Draw border
  R.Left := R.Left + Canvas.TextWidth(Caption) + 8;
  Y := (R.Bottom - R.Top) div 2;
  Canvas.Pen.Color := GetSysColor(COLOR_3DSHADOW);
  Canvas.MoveTo(R.Left,Y);
  Canvas.LineTo(R.Right, Y);
  Canvas.Pen.Color := GetSysColor(COLOR_3DHILIGHT);
  Canvas.MoveTo(R.Left, Y+1);
  Canvas.LineTo(R.Right, Y+1);
end;

// TATTitlePanel
//------------------------------------------------------------------------------
constructor TATTitlePanel.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
  Color := clWhite;
  Width := 400;
  Height := 60;
  FTitleLeft := 17;
  FTitleTop := 11;
  FCaptionLeft := 28;
  FCaptionTop := 30;
  FTitleFont := TFont.Create;
  FIcon := TIcon.Create;
  FIcontype := idCustom;
end;

destructor TATTitlePanel.Destroy;
begin
 inherited Destroy;
 FTitleFont.Free;
 FTitleFont := nil;
 FIcon.Free;
 FIcon      := nil;
end;

procedure TATTitlePanel.SetTitleFont(Value: TFont);
begin
  if FTitleFont <> Value then
  begin
    FTitleFont.Assign(Value);
    Invalidate;
  end;
end;

procedure TATTitlePanel.SetTitle(Value: string);
begin
  if FTitle <> Value then
  begin
    FTitle := Value;
    Invalidate;
  end;
end;

procedure TATTitlePanel.SetIcon(Value: TIcon);
begin
 FIcon.Assign(Value);
 Invalidate;
end;

procedure TATTitlePanel.SetTypeIcon(Value: TTitleIcons);
begin
  if (Value = idForm) and (Owner is TForm) and (not TForm(Owner).Icon.Empty) then
    FIcon.Assign(TForm(Owner).Icon)
  else if (Value = idApplication) then
    FIcon.Assign(Application.Icon)
  else if (Value = idCustom) then

  else
   Exit;

 FIconType := Value;
 Invalidate;
end;

procedure TATTitlePanel.Paint;
var Rec: TRect; R: Integer;
begin
  Rec := ClientRect;
  Canvas.Brush.Color := Self.Color;
  Canvas.FillRect(Rec);
  R := Rec.Bottom -2;
  Canvas.Pen.Color := GetSysColor(COLOR_3DSHADOW);
  Canvas.MoveTo(Rec.Left,R);
  Canvas.LineTo(Rec.Right, R);
  Canvas.Pen.Color := GetSysColor(COLOR_3DHILIGHT);
  Canvas.MoveTo(Rec.Left, R+1);
  Canvas.LineTo(Rec.Right, R+1);
  if not FIcon.Empty then
  begin
    R := (Clientheight - FIcon.height) div 2;
    Canvas.Draw(Rec.Right - FIcon.Width -8, R, FIcon);
  end;

  Canvas.Font.Assign(Self.FTitleFont);
  Canvas.Brush.Style := bsClear;
  Canvas.TextOut( FTitleLeft, FTitleTop, FTitle );

  Canvas.Font.Assign(Self.Font);
  Canvas.Brush.Style := bsClear;
  Canvas.TextOut( FCaptionLeft, FCaptionTop, Caption );
end;


procedure Register;
begin
 RegisterComponents('AT Standard',[TATLabel,
                                   TATTitlePanel]);
end;

end.
