unit ATXPtask;
{
  ATComponents 1.03  - XPTask, XPSmallButton, XPPanel, XPSectionLabel, XPHoverLabel
  Copyright (c) 2001 by Arpad Toth
  All rights reserved.

  All contents of this file and all other files included in the ATComponents
  archive  are   Copyright (c) 2001 Arpad Toth.  Use and/or  distribution of
  them requires acceptance of the author.
}


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, stdctrls, mmsystem, variants;

const
  cCloseUpAnim = 103;
  mxcolors=100;

type
  TXPStyle  = (xpWindows, xpOffice);
  TGrArray=array[1..mxcolors] of TColor;
  TXPDirect = (xdNone, xdLeft, xdRight, xdUp, xdDown, xdStretchImage,
               xdTileImage, xdSimpleImage);

  TXPSpeedButton = class(TGraphicControl)
  private
    FBmp    : TBitmap;
    FColor  : TColor;
    FColor1 : TColor;
    FIsLeave, FReMouseLeave,FMouseDown, FFrameDraw: Boolean;
    FOnMouseEnter, FOnMouseLeave: TNotifyEvent;
    procedure SetColor(Index: Integer; Value: TColor);
    procedure SetGlyph(Value: TBitmap);
  protected
    procedure Paint; override;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Tag;
    property Align;
    property Glyph: TBitmap read FBmp write SetGlyph;
    property Color: TColor index 0 read FColor write SetColor;
    property Frame: TColor index 1 read FColor1 write SetColor;
    property FrameDraw: Boolean read FFrameDraw write FFrameDraw;
    property Visible;
    property OnClick;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  end;

  THoverFont = class(TPersistent)
  private
   FFontColor: TColor;
  // FFontSize : Byte;
   FFontStyle: TFontStyles;
  public
  published
   property Color: TColor read FFontColor write FFontColor;
//   property Size: Byte read FFontSize write FFontSize;
   property Style: TFontStyles read FFontStyle write FFontStyle;
  end;

  TXPHoverLabel = class(TGraphicControl)
  private
    FColor, FColor1 : TColor;
    FFont: TFont;
    FGlyph: TBitmap;
    FHoverFont: THoverFont;
    FAlignment: TAlignment;
    Flayout: TTextLayout;
    FIsLeave, FReMouseLeave, FMouseDown, FWordWrap,
    FHoverCursor, FAutoSize, FAntialiased: Boolean;
    FOnMouseEnter, FOnMouseLeave: TNotifyEvent;
    function GetTransparent: Boolean;
//    function MouseInArea: Boolean
    procedure SetTransparent(const Value: Boolean);
    procedure SetColor(Index: Integer; Value: TColor);
    procedure SetFont(const Value: TFont);
    procedure SetHoverFont(const Value: THoverFont);
    procedure SetGlyph(const Value: TBitmap);
    procedure SetAlignment(const Value: TAlignment);
    procedure SetWordwrapt(const Value: Boolean);
    procedure SetLayout(const Value: TTextLayout);
    procedure SetAntiAliased(const Value: Boolean);
    procedure Changeantialiased;
    procedure WMSize(var message: TWMSize); message WM_SIZE;
    procedure CMTextChanged(var Msg: TMessage); message CM_TEXTCHANGED;
    procedure CMFontChanged(var Msg: TMessage); message CM_FONTCHANGED;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    procedure Paint; override;
    procedure BoundsAdjust;
    procedure Loaded; override;
    procedure SetAutoSize(Value: Boolean); override;
    procedure DoDrawText(var Rect: TRect; Flags: Longint);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Antialiased: Boolean read FAntialiased write SetAntiAliased;
  published
    property Align;
    property Tag;
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property AutoSize: Boolean read FAutoSize write SetAutoSize;
    property Anchors;
    property Caption;
    property Font: TFont read FFont write SetFont;
    property Glyph: TBitmap read FGlyph write SetGlyph;
    property Transparent: Boolean read GetTransparent write SetTransparent;
    property Color: TColor index 0 read FColor write SetColor;
    property HoverColor: TColor index 1 read FColor1 write SetColor;
    property Visible;
    property Layout: TTextLayout read Flayout write SetLayout;
    property HoverCursor: Boolean read FHoverCursor write FHoverCursor;
    property HoverFont: THoverFont read FHoverFont write SetHoverFont;
    property Wordwrap: Boolean read FWordWrap write SetWordwrapt;
    property ShowHint;
    property Enabled;
    property OnClick;
    property OnDblClick;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseHover: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  end;

  TXPSectionLabel = class(TGraphicControl)
  private
    FColor, FColor1 : TColor;
    FFrameDraw: Boolean;
    function GetTransparent: Boolean;
    procedure SetTransparent(Value: Boolean);
    procedure SetLinedraw(Value: Boolean);
    procedure SetColor(Index: Integer; Value: TColor);
    procedure WMSize(var message: TWMSize); message WM_SIZE;
    procedure CMTextChanged(var Msg: TMessage); message CM_TEXTCHANGED;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Tag;
    property Align;
    property Anchors;
    property Caption;
    property Font;
    property Transparent: Boolean read GetTransparent write SetTransparent;
    property Color: TColor index 0 read FColor write SetColor;
    property Line: TColor index 1 read FColor1 write SetColor;
    property LineDraw: Boolean read FFrameDraw write SetLineDraw;
    property Visible;
    property OnClick;
  end;

  TXPPanel = class(TCustomPanel)
  private
    FGa: TGrArray;
    FFGrgb,FORrgb: TColor;
    FDirect: TXPDirect;
    FMoveable: Boolean;
    FBGImage: TBitmap;
    procedure SetDirect(wDir: TXPDirect);
    procedure ChangeDir(Value: TXPDirect);
    procedure SetColor(Index: integer; Value: TColor);
    procedure SetBGImage(Value: TBitmap);
  protected
    procedure Paint; override;
    procedure ButtonDown(var Message: TWMLButtonDown);  Message WM_LBUTTONDOWN;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Caption;
    property Align;
    property Font;
    property BgImage: TBitmap read FBGImage write SetBGImage;
    property Moveable: Boolean read FMoveable write FMoveable default False;
    property Direction: TXPDirect read FDirect write ChangeDir;
    property Color:      TColor index 0 read FORrgb write SetColor;
    property ColorEnd:   TColor index 1 read FFGrgb write SetColor;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDrag;
  end;

  TXPTask = class;

  TTaskButton = class(TPersistent)
  private
    FOwner      : TXPTask;
    FVisible,
    FEnabled    : Boolean;
    FGlyph      : TBitmap;
    FOnClick    : TNotifyEvent;
    procedure SetGlyph(Value: TBitmap);
    procedure SetVisibleEnabled(Index: integer; Value: Boolean);
  protected
  public
    constructor Create(AOwner: TXPTask);
    destructor Destroy; override;
  published
    property Enabled: Boolean index 0 read FEnabled write SetVisibleEnabled;
    property Visible: Boolean index 1 read FVisible write SetVisibleEnabled;
    property Glyph: TBitmap read FGlyph write SetGlyph;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
  end;

  TTaskTitle = class(TPersistent)
  private
    FOwner      : TXPTask;
    FTitleText  : String;
    FTitleColor, FTitleColor1 : TColor;
    FHeigh      : Integer;
    FVisible    : Boolean;
    FFont       : TFont;
    FGlyphSpin, FGlyphClose: TTaskButton;
    procedure GlypsChange(Sender: TObject);
    procedure SetTaskButton(Index: Integer; Value: TTaskButton);
    procedure SetTitleHeight(const Value: Integer);
    procedure SetTitleFont(const Value: Tfont);
    procedure SetTitleColor(const Value: TColor);
    procedure SetTitleVisible(const Value: Boolean);
    procedure SetTitleCaption(const Value: string);
  protected
  public
    constructor Create(AOwner: TXPTask);
    destructor Destroy; override;
  published
    property ButtonSpin: TTaskButton index 0 read FGlyphSpin write SetTaskButton;
    property ButtonClose: TTaskButton index 1 read FGlyphClose write SetTaskButton;
    property Caption: String read FTitleText write SetTitleCaption;
    property Color: Tcolor read FTitleColor write SetTitleColor;
    property ColorIsMinimized: TColor read FTitleColor1 write FTitleColor1;
    property Height: Integer read FHeigh write SetTitleHeight;
    property Visible: Boolean read FVisible write SetTitleVisible;
    property Font: TFont read FFont write SetTitleFont;
  end;

  TXPTask = class(TCustomPanel)
  private
    FTransparent, Fmoveable, FSizeable, FIsMinimizedTask,
    FJustMinimize, FAnimated, FPlaySound: Boolean;
    FTitle: TTaskTitle;
    FFrameColor: TColor;
    FStyle: TXPStyle;
//    FBtn1, FBtn2: TBitmap;
    Fmarged, FOldHeight: integer;
    FSpinClick, FCloseClick, FTitleClick: TNotifyEvent;

    procedure SetTransparent(const Value: Boolean);
    procedure SetTitleParams(const Value: TTaskTitle);
    procedure SetTaskStyle(const Value: TXPStyle);
    procedure SetColorFrame(const Value: TColor);
    procedure SetSizeable(const Value: Boolean);
    procedure CollapseUp(const NewOld: Integer);
    procedure CollapseDown;
    procedure WMSize(var message: TWMSize); message WM_SIZE;
    procedure WMEraseBkGnd(var Message:TMessage); message WM_ERASEBKGND;
  protected
    procedure Paint; override;
    procedure PanelMove(var M: TWMNCHitTest); message WM_NCHITTEST;
    procedure ButtonDown(var Message: TWMLButtonDown);  Message WM_LBUTTONDOWN;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Show;
    procedure Hide;
    procedure MaximizeTask;
    procedure MinimizeTask;
    property IsMinimized: Boolean read FIsMinimizedTask;
  published
    property Align;
    property Anchors;
    property Animated: Boolean read FAnimated write FAnimated;
    property Color;
    property ColorFrame: TColor read FFrameColor write SetColorFrame;
    property Cursor;
    property Font;
    property Hint;
    property Moveable: Boolean read FMoveable write Fmoveable;
    property ParentShowHint;
    property Sound: Boolean read FPlaySound write FPlaySound;
    property PopupMenu;
    property Transparent: Boolean read FTransparent write SetTransparent;
    property Title: TTaskTitle read FTitle write SetTitleParams;
    property Sizeable: Boolean read FSizeable write SetSizeable default False;
    property ShowHint;
    property Style: TXPStyle read FStyle write SetTaskStyle;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnMouseMove;
    property OnMouseDown;
    property OnResize;
    property OnExit;
    property OnEnter;
    property OnSpinButtonClick: TNotifyEvent read FSpinClick write FSpinClick;
    property OnCloseButtonClick: TNotifyEvent read FCloseClick write FCloseClick;
    property OnTitleClick: TNotifyEvent read FTitleClick write FTitleClick;
  end;

function MouseInArea(Sender: TObject; R: TRect): Boolean; overload;
function MouseInArea(Sender: TObject): Boolean; overload;

procedure Register;

const
  Flags = SND_RESOURCE or SND_ASYNC;

implementation

{$R ATXPtask}
{$R EXTRASOUNDS}

uses ATFnclib;

function MouseInArea(Sender: TObject;R: TRect): Boolean;
var Pt: TPoint; Tc: TControl;
begin
  Tc := TControl(Sender);
  OffsetRect(R, Tc.ClientOrigin.X, Tc.ClientOrigin.Y);
  GetCursorPos(Pt);
  Result := PtInRect(R, Pt);
end;

function MouseInArea(Sender: TObject): Boolean;
var R: TRect; Pt: TPoint; Tc: TControl;
begin
  Tc := TControl(Sender);
  R := Tc.ClientRect;
  OffsetRect(R, Tc.ClientOrigin.X, Tc.ClientOrigin.Y);
  GetCursorPos(Pt);
  Result := PtInRect(R, Pt);
end;

constructor TXPSpeedButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque];
  FBmp := TBitmap.Create;
  FBmp.TransparentMode := tmAuto;
  FBmp.Transparent := True;
  SetBounds(0, 0, 16, 16);
  FColor   := clBtnFace;
  FColor1  := clWhite;
  FIsLeave := False;
  FFrameDraw := True;
  FReMouseLeave := False;
end;

destructor TXPSpeedButton.Destroy;
begin
 if assigned(fbmp) then fbmp.Free;
 inherited Destroy;
end;

procedure TXPSpeedButton.Paint;
var pt: TPoint; rect: TRect;
begin
//  inherited Paint;
  with Canvas do
   begin
      if FIsLeave then Brush.Color := FColor1
      else Brush.Color := FColor;
      rect := ClientRect;
      FillRect(rect);
      SetBkMode(Handle, Windows.TRANSPARENT);
//      FBmp.Transparent := True;
      pt := Point(0, (Height div 2) - (FBmp.Height div 2));
      if Assigned(fbmp) then
         Draw(pt.x, pt.y, FBmp)
   end;

     if FFrameDraw then
      if FIsLeave then
       Frame3D(self.Canvas, Rect, FColor, FColor, 1)
      else
       Frame3D(self.Canvas, Rect, FColor1, FColor1, 1);
end;

procedure TXPSpeedButton.SetColor(Index: integer;Value: TColor);
begin
  if index = 0 then FColor := Value else FColor1 := Value;
  Invalidate;   //?? Needed
end;

procedure TXPSpeedButton.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  FReMouseLeave := False;
  if not FMouseDown then
   begin
    if MouseInArea(Self) then FIsLeave := True;
    if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
    Invalidate;
   end else FReMouseLeave := True;
end;

procedure TXPSpeedButton.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  FReMouseLeave := False;
  if not FMouseDown then
   begin
    FIsLeave := False;
    if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
    Invalidate;
   end else FReMouseLeave := False;
end;

{function TXPSpeedButton.MouseInArea: Boolean;
var R: TRect; Pt: TPoint;
begin
  R := ClientRect;
  OffsetRect(R, ClientOrigin.X, ClientOrigin.Y);
  GetCursorPos(Pt);
  Result := PtInRect(R, Pt);
end;}

procedure TXPSpeedButton.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  FMouseDown := True;
  Invalidate;
end;

procedure TXPSpeedButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  FMouseDown := False;
  if FReMouseLeave then
   begin
    FIsLeave := False;
    if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
   end;
  Invalidate;
end;

procedure TXPSpeedButton.SetGlyph(Value: TBitmap);
begin
  FBmp.Assign(Value);
  if not FBmp.Empty then
   begin
   Visible := True;
   Invalidate;
   end else Visible := False;
end;

//----------------------------------------------------------
constructor TXPHoverLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle  := ControlStyle + [csOpaque];
  SetBounds(0, 0, 116, 19);
  FFont         := TFont.Create;
  FGlyph        := TBitmap.Create;
  FGlyph.Width  := 0;
  FGlyph.Height := 0;
  FGlyph.Transparent := true;
  FHoverFont := THoverFont.Create;
  with FHoverFont do
   begin
     FFontColor := Self.Font.Color;
//     FFontSize  := Self.Font.Size;
     FFontStyle := [];
   end;
  FColor     := clBtnFace;
  FColor1    := clBtnFace;
  FWordWrap  := False;
  FAutoSize  := False;
  Flayout    := tlTop;
  FAlignment := taLeftJustify;
  FAntialiased := True;
  ParentFont := True;
  Caption := Self.Name;
  Screen.Cursors[1] := LoadCursor(HInstance,'ATHANDS');
  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Width := 1;
end;

destructor TXPHoverLabel.Destroy;
begin
 Cursor := 0;
 FFont.free;
 if assigned(FGlyph) then FGlyph.Free;
 if assigned(FhoverFont) then FHoverFont.Free;
 inherited Destroy;
end;

procedure TXPHoverLabel.Loaded;
begin
  inherited Loaded;
  Changeantialiased;
  BoundsAdjust;
end;

procedure TXPHoverLabel.Changeantialiased;
//var lfn: TLogFont; isAnti: Boolean; hf: HFONT;
begin
{  GetObject(FFont.Handle, SizeOf(TLogFont), @lfn);
//  IsAnti := (lfn.lfQuality = ANTIALIASED_QUALITY);

  if FAntialiased then
     lfn.lfQuality := ANTIALIASED_QUALITY
  else lfn.lfQuality := DEFAULT_QUALITY;

  if FIsLeave then
    lfn.lfUnderline := Byte(fsUnderline in FHoverFont.Style);

  hf := CreateFontIndirect(lfn);
  FFont.Handle := hf;}
end;

procedure TXPHoverLabel.SetFont(const Value: TFont);
begin
 if (FFont<>Value) then
  begin
   FFont.Assign(Value);
   Changeantialiased;
   Invalidate;
  end;
end;

procedure TXPHoverLabel.BoundsAdjust;
const
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
var
  DC: HDC;
  X: Integer;
  Rec: TRect;
  AAlignment: TAlignment;
begin
  if not (csReading in ComponentState) and FAutoSize then
  begin
    Rec := ClientRect;
    DC := GetDC(0);
    Canvas.Handle := DC;
    DoDrawText(Rec, (DT_EXPANDTABS or DT_CALCRECT) or WordWraps[FWordWrap]);
    Canvas.Handle := 0;
    ReleaseDC(0, DC);
    X := Left;
    AAlignment := FAlignment;
    if UseRightToLeftAlignment then ChangeBiDiModeAlignment(AAlignment);
    if AAlignment = taRightJustify then Inc(X, Width - Rec.Right);
    if FGlyph <> nil then
       SetBounds(X, Top, Max(Rec.Right,FGlyph.Width+3+Rec.Right),Max(Rec.Bottom,FGlyph.Height))
    else
       SetBounds(X, Top, Rec.Right, Rec.Bottom);
  end;
end;

procedure TXPHoverLabel.DoDrawText(var Rect: TRect; Flags: Longint);
var Text: string;
begin
  Text := Caption;


  if (Flags and DT_CALCRECT <> 0) and ((Text = '') {or FShowAccelChar} and
    (Text[1] = '&') and (Text[2] = #0)) then Text := Text + ' ';
//  if not FShowAccelChar then Flags := Flags or DT_NOPREFIX;
  Flags := DrawTextBiDiModeFlags(Flags);

  Canvas.Font.Assign(FFont);

   if (FIsLeave and Enabled) then
    begin
     with Canvas.Font do
      begin
       Style := FHoverFont.Style;
       Color := FHoverFont.Color;
//       Size  := FHoverFont.Size;
      end;
     end;

  if not Enabled then
  begin
    OffsetRect(Rect, 1, 1);
    Canvas.Font.Color := clBtnHighlight;
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
    OffsetRect(Rect, -1, -1);
    Canvas.Font.Color := clBtnShadow;
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
  end
  else
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
end;

procedure TXPHoverLabel.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);

var Rec, CalcRect: TRect; DrawStyle: Longint;
begin

//  inherited Paint;
  if (csLoading in ComponentState) then exit;
  with Canvas do
   begin

    if FGlyph <> NIL then
     begin
       Draw(0,0, FGlyph);
       rec := Rect(FGlyph.Width+4,0,ClientRect.Right,ClientRect.Bottom);
     end else
       rec := Rect(0,0,ClientRect.Right,ClientRect.Bottom);

    if (not Transparent) then
     begin
      if FIsLeave and Enabled then Brush.Color := FColor1
      else Brush.Color := FColor;
      FillRect(rec);
     end;
    Brush.Style := bsClear;

    DrawStyle := DT_EXPANDTABS or WordWraps[FWordWrap] or Alignments[FAlignment];
    SetBkMode(Handle, Windows.TRANSPARENT);

    if FLayout <> tlTop then
    begin
      CalcRect := Rec;
      DoDrawText(CalcRect, DrawStyle or DT_CALCRECT);
      if FLayout = tlBottom then OffsetRect(Rec, 0, Height - CalcRect.Bottom)
      else OffsetRect(Rec, 0, (Height - CalcRect.Bottom) div 2);
    end;
    DoDrawText(Rec, DrawStyle);

   end;
end;

procedure TXPHoverLabel.CMTextChanged(var Msg: TMessage);
begin
  Invalidate;
  BoundsAdjust;
end;

procedure TXPHoverLabel.CMFontChanged(var Msg: TMessage);
begin
//  ApplyAntialiasing;
  Inherited;
  Invalidate;
end;

procedure TXPHoverLabel.WMSize(var Message: TWMSize);
begin
  inherited;
  Invalidate;
//  SendMessage(Canvas.Handle, WM_PAINT, 0, 0);
end;

procedure TXPHoverLabel.SetColor(Index: Integer; Value: TColor);
begin
  case index of
  0: FColor := Value;
  1: FColor1:= Value;
  end;
  invalidate;
end;

procedure TXPHoverLabel.SetTransparent(const Value: Boolean);
begin
  if Transparent <> Value then
  begin
    if Value then
      ControlStyle := ControlStyle - [csOpaque] else
      ControlStyle := ControlStyle + [csOpaque];
    Invalidate;
  end;
end;

function TXPHoverLabel.GetTransparent: Boolean;
begin
  Result := not (csOpaque in ControlStyle);
end;

procedure TXPHoverLabel.SetLayout(const Value: TTextLayout);
begin
  if FLayout <> Value then
  begin
    FLayout := Value;
    Invalidate;
  end;
end;

procedure TXPHoverLabel.SetAutoSize(Value: Boolean);
begin
  if FAutoSize <> Value then
  begin
    FAutoSize := Value;
    BoundsAdjust;
  end;
end;

{procedure TXPHoverLabel.ApplyAntialiasing;
var lf: TLogFont; IsAnti: Boolean;
begin
    GetObject(Self.Font.Handle, SizeOf(TLogFont), @lf);
    IsAnti := (lf.lfQuality = ANTIALIASED_QUALITY);
    if IsAnti <> FAntialiased then
      begin
         if FAntialiased then
           begin
            lf.lfQuality := ANTIALIASED_QUALITY;
           end
         else
           begin
            lf.lfQuality := DEFAULT_QUALITY;
           end;
         Self.Font.Handle := CreateFontIndirect(lf);
        end;
end;}

procedure TXPHoverLabel.SetAntiAliased(const Value: Boolean);
begin
   FAntialiased := Value;
//   Changeantialiased;
   Invalidate;
end;

procedure TXPHoverLabel.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  FReMouseLeave := False;
  if not FMouseDown then
   begin
    if MouseInArea(Self) then FIsLeave := True;
    if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
    if FHoverCursor and Enabled then Cursor := 1;
    Invalidate;
   end else FReMouseLeave := True;
end;

procedure TXPHoverLabel.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  FReMouseLeave := False;
  if not FMouseDown then
   begin
    FIsLeave := False;
    if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
    if FHoverCursor and Enabled then Cursor := 0;
    Invalidate;
   end else FReMouseLeave := False;
end;

procedure TXPHoverLabel.Sethoverfont(const Value: THoverFont);
begin
 if (FHoverFont <> Value) then
  FHoverFont := Value;
end;

procedure TXPHoverLabel.SetGlyph(const Value: TBitmap);
begin
 if (FGlyph <> Value) then
  begin
   FGlyph.Assign(Value);
   FGlyph.Transparent := true;
   Invalidate;
  end;
end;

procedure TXPHoverLabel.SetAlignment(const Value: Talignment);
begin
 if (FAlignment <> Value) then
  begin
   FAlignment := Value;
   Invalidate;
  end;
end;

procedure TXPHoverLabel.SetWordwrapt(const Value: Boolean);
begin
 if (FWordWrap <> Value) then
  begin
   FWordWrap := Value;
   Invalidate;
  end;
end;

//----------------------------------------------------------
constructor TXPSectionLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque];
  SetBounds(0, 0, 116, 19);
  FColor   := clBtnFace;
  FColor1  := clGray;
  Font.Style := [fsBold];
  FFrameDraw := True;
  Caption := Self.Name;
  Transparent := True;
  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Width := 1;
end;

destructor TXPSectionLabel.Destroy;
begin
 inherited Destroy;
end;

procedure TXPSectionLabel.Paint;
var x,y: integer; rec: TRect;
begin
  Height := Canvas.TextHeight('^_') + 2;

  inherited Paint;
  with Canvas do
   begin
    rec := ClientRect;  //GetClientRect;
    if not Transparent then
     begin
      Brush.Color := FColor;
      FillRect(rec);
     end;
    Brush.Style := bsClear;

    SetBkMode(Handle, Windows.TRANSPARENT);
    Font := Self.Font;
    x:=0;
    y:=height div 2 - canvas.textHEIGHT('^_') div 2;
    Textout(x,y-1,caption);
    if FFrameDraw then
     begin
       Pen.Color := Line;
       y:=Rec.Bottom-1;
       MoveTo(Rec.Left,y);
       LineTo(Rec.Right,y);
     end;

   end;
end;

procedure TXPSectionLabel.CMTextChanged(var Msg: TMessage);
begin inherited; invalidate; end;

procedure TXPSectionLabel.WMSize(var Message: TWMSize);
begin
  inherited;
  SendMessage(Canvas.Handle, WM_PAINT, 0, 0);
end;

procedure TXPSectionLabel.SetColor(Index: Integer; Value: TColor);
begin
  case index of
  0: FColor := Value;
  1: FColor1:= Value;
  end;
  invalidate;
end;

procedure TXPSectionLabel.SetTransparent(Value: Boolean);
begin
  if Transparent <> Value then
  begin
    if Value then
      ControlStyle := ControlStyle - [csOpaque] else
      ControlStyle := ControlStyle + [csOpaque];
    Invalidate;
  end;
end;

procedure TXPSectionLabel.SetLinedraw(Value: Boolean);
begin
 FFrameDraw := Value;
 Invalidate;
end;


function TXPSectionLabel.GetTransparent: Boolean;
begin
  Result := not (csOpaque in ControlStyle);
end;

//----------------------------------------------------------
constructor TXPPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Parent      := TWinControl(AOwner);
  Alignment   := taLeftJustify;
  BevelInner  := bvNone;
  BevelOuter  := bvNone;
  BorderWidth := 0;
  FORrgb      := clHighlight;
  FFGrgb      := clWhite;
  FDirect     := xdNone;
  FBGImage    := TBitmap.Create;
end;

destructor TXPPanel.Destroy;
begin
 FBGImage.Free;
 inherited destroy;
end;

procedure TXPPanel.ButtonDown(var Message: TWMLButtonDown);
begin
  inherited;
  if FMoveable then
   begin
    releasecapture;
    sendmessage(Parent.Handle, WM_NCLBUTTONDOWN, 2, 0);
   end;
end;

procedure TXPPanel.Paint;
var x,y: integer;
begin
   Canvas.Font.Assign(font);
   Canvas.Pen.Style := psSolid;
   Canvas.Pen.Mode  := pmCopy;
   inherited paint;
   SetDirect(FDirect);
   setBkMode(canvas.handle,1);
   x:=borderwidth+bevelwidth+7;
   y:=height div 2 - canvas.textHEIGHT('^_') div 2;
   canvas.textout(x,y-1,caption);
end;

procedure TXPPanel.SetBGImage(Value: TBitmap);
begin
 FBGImage.Assign(Value);
 if (BGImage.Empty) or (BGImage = nil) then
  begin
   FDirect := xdNone;
   Invalidate;
  end;
end;

procedure TXPPanel.ChangeDir(Value: TXPDirect);
begin
  FDirect := Value;
  invalidate;
end;

procedure TXPPanel.SetColor(Index: integer; Value: TColor);
begin
  case Index of
   0: if FORrgb <> Value then FORrgb := Value;
   1: if FFGrgb <> Value then FFGrgb := Value;
  end;
   invalidate;
end;


procedure TXPPanel.SetDirect(wDir: TXPDirect);

   procedure CalcColGradients(col1,col2:TColor; n:Integer);
   var i:integer;
       ar,ag,ab,br,bb,bg:integer;
       kr,kg,kb:real;
   begin
     ar:=GetRValue(col1);br:=GetRValue(col2);
     AG:=GetGValue(col1);bG:=GetGValue(col2);
     AB:=GetBValue(col1);bB:=GetBValue(col2);
     kr:=(br-ar) /n;
     kg:=(bg-ag) /n;
     kb:=(bb-ab) /n;
     for i:=0 to n-1 do
      FGa[i+1]:=rgb(trunc(ar + i*kr),trunc(ag+i*kg),trunc(ab+i*kb));
   end;

   procedure VertDirect(Banda: TRect);
   var I : Integer;
   begin
   Banda.Top := 0;
   Banda.Bottom := Height;
    with canvas do
    for I := 0 to mxcolors-1 do
    begin
       Banda.Left  := MulDiv (I    , Width, mxcolors);
       Banda.Right := MulDiv (I + 1, Width, mxcolors);
       Brush.Color := FGa[i+1];
       FillRect (Banda);
    end;
   end;

   procedure HoriDirect(Banda: TRect);
   var I : Integer;
   begin
   Banda.Left := 0;
   Banda.Right := Width;
    with canvas do
    for I := 0 to mxcolors-1 do
    begin
       Banda.Top    := MulDiv (I    , Height, mxcolors);
       Banda.Bottom := MulDiv (I + 1, Height, mxcolors);
       Brush.Color  := FGa[i+1];
       FillRect (Banda);
    end;
   end;

   procedure BGImages(wDir: TXPDirect);
   var i,j,k:integer;
    x,y:integer;
   begin
      k:=1;
      if wDir = xdStretchImage then
         canvas.StretchDraw(rect(k,k,width-k-1,height-k-1),FBGImage)
      else if wDir = xdTileImage then
      begin
          x:=1+(width-2*k) div FBGImage.width;
          y:=1+(height-2*k) div FBGImage.height;
          for i:=1 to x do for j:=1 to y do
	  canvas.Draw(k+FBGImage.width*(i-1),k+FBGImage.height*(j-1),FBGImage);
      end
      else canvas.Draw(k,k,FBGImage);
   end;

var Banda : TRect;
begin
   case wDir of
   xdNone:  with canvas do
             begin
              Banda.Left := 0; Banda.Right := Width;
              Banda.top:=0   ; Banda.bottom:= Height;
              Brush.Color := FORrgb;
              FillRect(Banda);
             end;
   xdLeft,xdRight:
           begin
            if wDir = xdRight then
            CalcColGradients(FORrgb, FFGrgb, mxcolors) else
            CalcColGradients(FFGrgb, FORrgb, mxcolors);
            VertDirect(Banda);
           end;
   xdUp,xdDown:
           begin
            if wDir = xdDown then
            CalcColGradients(FORrgb, FFGrgb, mxcolors) else
            CalcColGradients(FFGrgb, FORrgb, mxcolors);
            HoriDirect(Banda);
           end;
   xdStretchImage, xdTileImage, xdSimpleImage:
           begin
            SetDirect(xdNone);
            if not(FBGImage = nil) or not(FBGImage.Empty) then BGImages(wDir);
           end;
   end;
end;

//----------------------------------------------------------
constructor TTaskButton.Create(AOwner: TXPTask);
begin
 inherited Create;
 FOwner := AOwner;
 FGlyph := TBitmap.Create;
 FGlyph.Transparent := True;
end;

destructor TTaskButton.Destroy;
begin
 FGlyph.Free;
 inherited;
end;

procedure TTaskButton.SetGlyph(Value: Tbitmap);
begin
 if FGlyph <> Value then
  begin
   FGlyph.Assign(Value);
   FGlyph.Transparent := True;
   FOwner.Invalidate;
  end;
end;

procedure TTaskButton.SetVisibleEnabled(Index: integer; Value: Boolean);
begin
 case index of
  0: if (FEnabled <> Value) then
     begin
      FEnabled := Value;
      FOwner.Invalidate;
     end;
  1: if (FVisible <> Value) then
     begin
      FVisible := Value;
      FOwner.Invalidate;
     end;
 end;
end;

//------------------
constructor TTaskTitle.Create(AOwner: TXPTask);
begin
 inherited Create;
 FOwner := AOwner;
 FFont  := TFont.Create;
 FGlyphSpin  := TTaskButton.Create(AOwner);
 FGlyphSpin.Glyph.OnChange  := GlypsChange;
 FGlyphClose := TTaskButton.Create(AOwner);
 FGlyphClose.Glyph.OnChange := GlypsChange;
end;

destructor TTaskTitle.Destroy;
begin
 FGlyphSpin.Free;
 FGlyphClose.Free;
 FFont.Free;
 inherited;
end;

procedure TTaskTitle.GlypsChange(Sender: TObject);
begin
 with FGlyphSpin.Glyph do
  if Empty then LoadFromResourceName(HInstance,'BM_SPIN16');
 with FGlyphClose.Glyph do
  if Empty then LoadFromResourceName(HInstance,'BM_CLOSE16');
end;


procedure TTaskTitle.SetTitleHeight(const Value: Integer);
begin
 if FHeigh <> Value then
  begin
   FHeigh := Value;
   FOwner.Invalidate;
  end;
end;

procedure TTaskTitle.SetTitleVisible(const Value: Boolean);
begin
 if FVisible <> Value then
  begin
   FVisible := Value;
   FOwner.Invalidate;
  end;
end;

procedure TTaskTitle.SetTitleFont(const Value: TFont);
begin
 if (FFont <> Value) then
  begin
   FFont.Assign(Value);
   FOwner.Invalidate;
  end;
end;

procedure TTaskTitle.SetTaskButton(Index: Integer; Value: TTaskButton);
begin
 inherited;
 case Index of
  0: begin
      if FGlyphSpin = Value then exit;
      FGlyphSpin  := Value;
{       with FGlyphSpin.Glyph do
         if Empty then LoadFromResourceName(HInstance,'BM_SPIN16');
      FOwner.FTitle.Height := Max( FOwner.FTitle.Height,FGlyphSpin.Glyph.Height);}
      FOwner.Invalidate;
     end;
  1: begin
      if FGlyphClose = Value then exit;
      FGlyphClose := Value;
{       with FGlyphClose.Glyph do
         if Empty then LoadFromResourceName(HInstance,'BM_CLOSE');}
      FOwner.FTitle.Height := Max( FOwner.FTitle.Height,FGlyphClose.Glyph.Height);
      FOwner.Invalidate;
//      inherited;
     end;
 end;
end;

procedure TTAskTitle.SetTitleColor(const Value: TColor);
begin
 if (FTitleColor <> Value) then
  begin
   FTitleColor := Value;
   FOwner.Invalidate;
  end;
end;

procedure TTAskTitle.SetTitleCaption(const Value: string);
begin
 if (FTitleText <> Value) then
  begin
   FTitleText := Value;
   FOwner.Invalidate;
  end;
end;

constructor TXPTask.Create(AOwner: TComponent);
begin
  inherited;
  Color := $00EAEAEA;
//  FBtn1 := TBitmap.create;
//  FBtn2 := TBitmap.Create;
  FFrameColor := clWindow;
  ParentColor := False;
  ParentFont := True;
  ControlStyle := ControlStyle + [csAcceptsControls, csOpaque];
  SetBounds(0, 0, 185, 41);
  FOldHeight  := Height;
  FMarged       := 0;
  FJustMinimize := False;
  FTitle := TTaskTitle.Create(Self);
  with FTitle do
   begin
    Height := 19;
    Color  := $00CED6D6;
    ColorIsMinimized    := $00EAEAEA;
    FGlyphSpin.OnClick  := FSpinClick;
    FGlyphSpin.Glyph.LoadFromResourceName(HInstance,'BM_SPIN16');
//    FBtn1.Assign(FGlyphSpin.Glyph);
    FGlyphClose.OnClick := FCloseClick;
    FGlyphClose.Glyph.LoadFromResourceName(HInstance,'BM_CLOSE16');
//    FBtn2.Assign(FGlyphClose.Glyph);
   end;
   DoubleBuffered := True;
end;

destructor TXPTask.Destroy;
begin
// if Assigned(Fbtn1) then FBtn1.Free;
// if assigned(Fbtn2) then FBtn2.Free;
 FTitle.Free;
 inherited Destroy;
end;

procedure TXPTask.Hide;
begin Visible := False; end;

procedure TXPTask.Show;
begin Visible := True;  end;

procedure TXPTask.MaximizeTask;
begin
 if (csLoading in ComponentState) or (not FIsMinimizedTask) then Exit;
 if FPlaySound then PlaySound('SNF_005',0,Flags);
 FJustMinimize := True;
 if not FAnimated then Height := FOldHeight else CollapseDown;
 FIsMinimizedTask := False;
 FJustMinimize := False;
end;

procedure TXPTask.MinimizeTask;
var Newold: integer;
begin
 if (csLoading in ComponentState) or FIsMinimizedTask then Exit;
 if not (Align in [alNone,alTop,alBottom]) then exit;
 Newold := FOldHeight;
 FJustMinimize := True;
 if not FAnimated then Height := FTitle.Height else CollapseUp(Newold);
 if FPlaySound then PlaySound('SNF_001',0,Flags);
 FOldHeight := Newold;
 FJustMinimize := False;
 FIsMinimizedTask := True;
end;

procedure TXPTask.CollapseUp(const NewOld: integer);
var I:Integer;
begin

if (csLoading in ComponentState) then exit;
 I := NewOld;
 repeat
  Dec(I);
  if (I mod 4)=0 then
   begin
    Height := I;
    Application.ProcessMessages;
   end;
 until (I < FTitle.Height);
 Height:=FTitle.Height;
 Invalidate;
end;

procedure TXPTask.CollapseDown;
var I:Integer;
begin
 if (csLoading in ComponentState) then exit;
 I := FTitle.Height;
 repeat
  Inc(I);
  if (I mod 4)=0 then
   begin
    Height := I;
    Application.ProcessMessages;
   end;
 until (I > FOldHeight);
 Height:= FOldHeight;
 invalidate;
end;

procedure TXPTask.SetTitleParams(const Value: TTaskTitle);
begin
 if FTitle <> Value then
  begin
   FTitle := Value;
   Invalidate;
  end;
end;

procedure TXPTask.SetTransparent(const Value: Boolean);
begin
 if FTransparent <> Value then
  begin
   FTransparent := Value;
   Invalidate;
  end;
end;

procedure TXPTask.SetTaskStyle(const Value: TXPStyle);
begin
 if FStyle <> Value then
  begin
   FStyle := Value;
   Invalidate;
  end;
end;

procedure TXPTask.Paint;

  procedure DrawParentImg(Control: TControl; Dest: TCanvas);
  var SaveIndex: Integer; DC: HDC; Position: TPoint;
  begin
    with Control do
     begin
       if Parent = nil then Exit;
         DC := Dest.Handle;
         SaveIndex := SaveDC(DC);
         GetViewportOrgEx(DC, Position);
         SetViewportOrgEx(DC, Position.X - Left, Position.Y - Top, nil);
         IntersectClipRect(DC, 0, 0, Parent.ClientWidth, Parent.ClientHeight);
         Parent.Perform(WM_ERASEBKGND, DC, 0);
         Parent.Perform(WM_PAINT, DC, 0);
         RestoreDC(DC, SaveIndex);
     end;
  end;

var  memoryBitmap, btnBtimap: TBitmap; r, rr: TRect;
     y,z: integer;
begin

  memoryBitmap := TBitmap.Create; // create memory-bitmap to draw
  btnBtimap    := TBitmap.Create;

  try
    memoryBitmap.Height := ClientRect.Bottom;
    memoryBitmap.Width := ClientRect.Right;
    // Draw Background
    r := ClientRect;
    rr := r;
   if FTitle.Visible then
    if not (FStyle in [xpOffice]) then
        rr := Rect(R.Left,R.Top+8,R.Right,R.Bottom);

      DrawParentImg(Self, memoryBitmap.Canvas);

    if not FTransparent then
     with memoryBitmap.Canvas do
     begin
      Brush.Color := Self.Color;
      FillRect(rr);
     end;

    Frame3D(memoryBitmap.Canvas, rr, FFrameColor, FFrameColor, 1);

    // Draw Border
    if FTitle.Visible then
     begin
       if not (FStyle in [xpOffice]) then Inflaterect(r,2,2) else
       InflateRect(r,-1,-1);
       with memoryBitmap.Canvas do
        begin

        if not FIsMinimizedTask then
         begin
          Brush.Color := FTitle.Color;
          Pen.Color   := FTitle.Color;
         end
        else
         begin
          Brush.Color := FTitle.ColorIsMinimized;
          Pen.Color   := FTitle.ColorIsMinimized;
         end;

        if not (FStyle in [xpOffice]) then
         begin
          RoundRect(0,0,Width,FTitle.height,8,8);
          Rectangle(r.left,r.top+10,r.right,r.top+FTitle.Height+4 - 2 * BevelWidth);
          end
        else
          Rectangle(r.left,r.top,r.right,r.top+FTitle.Height- 2 * BevelWidth);

         Font.Assign(FTitle.Font);
         y := FTitle.Height div 2 - TextHeight('^_') div 2;
        end;

       TextOut(memoryBitmap.Canvas.Handle,r.Left+5,y-1,pchar(FTitle.Caption),length(FTitle.Caption));
       // -------------- Drawing title buttons
       // Close button
       z := 0;

       if FTitle.FGlyphClose.Visible then
        begin
         btnBtimap.Assign(FTitle.FGlyphClose.Glyph);
         z := btnBtimap.Width;
         if not FTitle.FGlyphClose.Enabled then btnBtimap := InvertBmp(btnBtimap);
            memoryBitmap.Canvas.Draw(rr.Right-z-1, (FTitle.Height div 2 - btnBtimap.Height div 2) ,btnBtimap);
       end;

        // Spin button
       btnBtimap.FreeImage;
       if FTitle.FGlyphSpin.Visible then
        begin
         btnBtimap.Assign(FTitle.FGlyphSpin.Glyph);
         if not FTitle.FGlyphSpin.Enabled then btnBtimap := InvertBmp(btnBtimap);
         memoryBitmap.Canvas.Draw(rr.Right-btnBtimap.Width-z-1,(FTitle.Height div 2 - btnBtimap.Height div 2),btnBtimap);

        end;
//       R.top := R.Top + FTitle.Height - 4 * BevelWidth;
     end;
    // Copy memoryBitmap to screen
    Canvas.CopyRect(ClientRect, memoryBitmap.canvas, ClientRect);

  finally
    FreeAndNil(btnBtimap);
    FreeAndNil(memoryBitmap);
//    memoryBitmap.free; // delete the bitmaps
  end;
end;

procedure TXPTask.MouseDown(Button: TMouseButton; Shift: TShiftState; X,Y: Integer);
var r: TRect; W,W1: Integer;

begin
  inherited;
  if (csDesigning in ComponentState) or FJustMinimize then Exit;
  W := 0;
  if FTitle.Visible then
  begin
    r := ClientRect;

    if FTitle.ButtonClose.Visible then
     begin
      W := FTitle.ButtonClose.Glyph.Width;

      if PtInRect(rect(r.Right-2-W,r.Top+2,r.Right-2,r.Top+2+W),point(x,y)) and
         FTitle.ButtonClose.Enabled then
       begin
         if Assigned(FCloseClick) then FCloseClick(Self) else
            Hide;
       end;
     end;

    if FTitle.ButtonSpin.Visible then
     begin
      W := W + FTitle.ButtonSpin.Glyph.Width;
      W1 := FTitle.ButtonSpin.Glyph.Width;

      if PtInRect(rect(r.Right-2-W,r.Top+2,r.Right-2-W+W1,r.Top+W1+2),point(x,y)) and
         FTitle.ButtonSpin.Enabled then
       begin
        if Assigned(FSpinClick) then FSpinClick(Self) else
         if FIsMinimizedTask then MaximizeTask else
            MinimizeTask;
       end;
     end;

     if PtInRect(rect(r.left,r.Top,r.Right-2,r.Top + FTitle.Height),point(x,y)) then
      begin
       if Assigned(FTitleClick) then
         FTitleClick(Self);
      end;
  end;
end;

procedure TXPTask.ButtonDown(var Message: TWMLButtonDown);
var rr: TRect; rr1: integer;
begin
  inherited;
  rr := ClientRect; rr1:=0;
  if (csDesigning in ComponentState) or FJustMinimize then exit;
  if (Align in [alClient, alLeft, alTop, alRight, alBottom]) then exit;
  if FTitle.Visible then
    if FMoveable then
    begin
     if FTitle.ButtonSpin.Visible then rr1 := FTitle.ButtonSpin.Glyph.Width;
     if FTitle.ButtonClose.Visible then rr1 := rr1 + FTitle.ButtonClose.Glyph.Width;
     if MouseInArea(Self, Rect(RR.Left,rr.Top,rr.Right-rr1-2,rr.Top+FTitle.Height) ) then
      begin
       ReleaseCapture;
       SendMessage(Handle, WM_NCLBUTTONDOWN, 2, 0);
      end;
    end;
end;

procedure TXPTask.PanelMove(var M: TWMNCHitTest);
var p: Tpoint;
begin
 if (csDesigning in ComponentState) then begin
    parent.dispatch(m);
    exit;
 end;

 if (FSizeable and not IsMinimized) then
 begin
  p.x:=m.XPos; p.y:=m.YPos;
  p:=screentoclient(p);

    if ((width- p.x<FMarged) and (height-p.y<FMarged)) then
    m.result:=HTBOTTOMRIGHT  else
    if (width- p.x<FMarged) then m.result:=HTRIGHT else
//    if (p.y<FMarged) then m.result:=HTTOP else
    if (height-p.y<FMarged) then m.result:=HTBOTTOM
    else m.result:=HTCLIENT; // HTCAPTION
 end else inherited;
end;

procedure TXPTask.WMSize(var Message: TWMSize);
begin
  if not IsMinimized then
   begin
    if (not FJustMinimize) and (Message.Height >= FTitle.Height) then inherited;
    if Message.Height < FTitle.Height then Height := FTitle.Height;
    FOldHeight := Height;
   end;
   SendMessage(Canvas.Handle, WM_PAINT, 0, 0);
end;

procedure TXPTask.WMEraseBkGnd(var Message: TMessage);
begin
 inherited;
end;

procedure TXPTask.SetSizeable(const Value: Boolean);
begin
 if (FSizeable <> Value) then
  begin
    if FSizeable then FMarged := 0 else FMarged := 5;
    FSizeable := Value;
  end;
end;

procedure TXPTask.SetColorFrame(const Value: TColor);
begin
 if (FFrameColor <> Value) then
  begin
    FFrameColor := Value;
    Invalidate;
  end;
end;

procedure Register;
begin
 RegisterComponents('AT Standard',[TXPSectionLabel, TXPHoverLabel,
                                   TXPSpeedButton,
                                   TXPPanel, TXPTask]);
end;


end.
